@@ -22,6 +22,7 @@ import os
 
 import netaddr
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import requests
 import six
 import six.moves.http_client as httplib
@@ -29,6 +30,7 @@ import suds
 from suds import cache
 from suds import client
 from suds import plugin
+import suds.sax.element as element
 from suds import transport
 
 from oslo_vmware._i18n import _
@@ -48,6 +50,9 @@ LOG = logging.getLogger(__name__)
 class ServiceMessagePlugin(plugin.MessagePlugin):
     """Suds plug-in handling some special cases while calling VI SDK."""
 
+    # list of XML elements which are allowed to be empty
+    EMPTY_ELEMENTS = ["VirtualMachineEmptyProfileSpec"]
+
     def add_attribute_for_value(self, node):
         """Helper to handle AnyType.
 
@@ -58,6 +63,22 @@ class ServiceMessagePlugin(plugin.MessagePlugin):
         """
         if node.name == 'value' or node.name == 'val':
             node.set('xsi:type', 'xsd:string')
+        # removeKey may be a 'int' or a 'string'
+        if node.name == 'removeKey':
+            try:
+                int(node.text)
+                node.set('xsi:type', 'xsd:int')
+            except (ValueError, TypeError):
+                node.set('xsi:type', 'xsd:string')
+
+    def prune(self, el):
+        pruned = []
+        for c in el.children:
+            self.prune(c)
+            if c.isempty(False) and c.name not in self.EMPTY_ELEMENTS:
+                pruned.append(c)
+        for p in pruned:
+            el.children.remove(p)
 
     def marshalled(self, context):
         """Modifies the envelope document before it is sent.
@@ -70,7 +91,8 @@ class ServiceMessagePlugin(plugin.MessagePlugin):
         # Suds builds the entire request object based on the WSDL schema.
         # VI SDK throws server errors if optional SOAP nodes are sent
         # without values; e.g., <test/> as opposed to <test>test</test>.
-        context.envelope.prune()
+
+        self.prune(context.envelope)
         context.envelope.walk(self.add_attribute_for_value)
 
 
@@ -130,11 +152,13 @@ class LocalFileAdapter(requests.adapters.HTTPAdapter):
 
     def send(self, request, stream=False, timeout=None,
              verify=True, cert=None, proxies=None):
+        """Sends request for a local file."""
         return self._build_response_from_file(request)
 
 
 class RequestsTransport(transport.Transport):
-    def __init__(self, cacert=None, insecure=True, pool_maxsize=10):
+    def __init__(self, cacert=None, insecure=True, pool_maxsize=10,
+                 connection_timeout=None):
         transport.Transport.__init__(self)
         # insecure flag is used only if cacert is not
         # specified.
@@ -142,7 +166,10 @@ class RequestsTransport(transport.Transport):
         self.session = requests.Session()
         self.session.mount('file:///',
                            LocalFileAdapter(pool_maxsize=pool_maxsize))
+        self.session.mount('https://', requests.adapters.HTTPAdapter(
+            pool_connections=pool_maxsize, pool_maxsize=pool_maxsize))
         self.cookiejar = self.session.cookies
+        self._connection_timeout = connection_timeout
 
     def open(self, request):
         resp = self.session.get(request.url, verify=self.verify)
@@ -152,7 +179,8 @@ class RequestsTransport(transport.Transport):
         resp = self.session.post(request.url,
                                  data=request.message,
                                  headers=request.headers,
-                                 verify=self.verify)
+                                 verify=self.verify,
+                                 timeout=self._connection_timeout)
         return transport.Reply(resp.status_code, resp.headers, resp.content)
 
 
@@ -188,18 +216,24 @@ class Service(object):
     """
 
     def __init__(self, wsdl_url=None, soap_url=None,
-                 cacert=None, insecure=True, pool_maxsize=10):
+                 cacert=None, insecure=True, pool_maxsize=10,
+                 connection_timeout=None, op_id_prefix='oslo.vmware'):
         self.wsdl_url = wsdl_url
         self.soap_url = soap_url
+        self.op_id_prefix = op_id_prefix
         LOG.debug("Creating suds client with soap_url='%s' and wsdl_url='%s'",
                   self.soap_url, self.wsdl_url)
-        transport = RequestsTransport(cacert, insecure, pool_maxsize)
+        transport = RequestsTransport(cacert=cacert,
+                                      insecure=insecure,
+                                      pool_maxsize=pool_maxsize,
+                                      connection_timeout=connection_timeout)
         self.client = client.Client(self.wsdl_url,
                                     transport=transport,
                                     location=self.soap_url,
                                     plugins=[ServiceMessagePlugin()],
                                     cache=_CACHE)
         self._service_content = None
+        self._vc_session_cookie = None
 
     @staticmethod
     def build_base_url(protocol, host, port):
@@ -251,6 +285,25 @@ class Service(object):
                                                fault_string,
                                                details=details)
 
+    def _set_soap_headers(self, op_id):
+        """Set SOAP headers for the next remote call to vCenter.
+
+        SOAP headers may include operation ID and vcSessionCookie.
+        The operation ID is a random string which allows to correlate log
+        messages across different systems (OpenStack, vCenter, ESX).
+        vcSessionCookie is needed when making PBM calls.
+        """
+        headers = []
+        if self._vc_session_cookie:
+            elem = element.Element('vcSessionCookie').setText(
+                self._vc_session_cookie)
+            headers.append(elem)
+        if op_id:
+            elem = element.Element('operationID').setText(op_id)
+            headers.append(elem)
+        if headers:
+            self.client.set_options(soapheaders=headers)
+
     @property
     def service_content(self):
         if self._service_content is None:
@@ -288,6 +341,19 @@ class Service(object):
                                                         managed_object)
                 if managed_object is None:
                     return
+
+                skip_op_id = kwargs.pop('skip_op_id', False)
+                op_id = None
+                if not skip_op_id:
+                    # Generate opID. It will appear in vCenter and ESX logs for
+                    # this particular remote call.
+                    op_id = '%s-%s' % (self.op_id_prefix,
+                                       uuidutils.generate_uuid())
+                    LOG.debug('Invoking %s.%s with opID=%s',
+                              managed_object._type,
+                              attr_name,
+                              op_id)
+                self._set_soap_headers(op_id)
                 request = getattr(self.client.service, attr_name)
                 response = request(managed_object, **kwargs)
                 if (attr_name.lower() == 'retrievepropertiesex'):
@@ -315,7 +381,12 @@ class Service(object):
                 if detail:
                     for fault in detail.getChildren():
                         fault_type = fault.get('type')
-                        if fault_type.endswith(exceptions.SECURITY_ERROR):
+                        # NOTE(vbala): PBM faults use vim25 namespace. Also,
+                        # PBM APIs throw NotAuthenticated in vSphere 6.5 for
+                        # session expiry.
+                        if (fault_type.endswith(exceptions.SECURITY_ERROR) or
+                                fault_type.endswith(
+                                    exceptions.NOT_AUTHENTICATED)):
                             fault_type = exceptions.NOT_AUTHENTICATED
                         fault_list.append(fault_type)
                         for child in fault.getChildren():
