@@ -43,11 +43,12 @@ class DatastoreTestCase(base.TestCase):
 
     def test_ds(self):
         ds = datastore.Datastore(
-            "fake_ref", "ds_name", 2 * units.Gi, 1 * units.Gi)
+            "fake_ref", "ds_name", 2 * units.Gi, 1 * units.Gi, 1 * units.Gi)
         self.assertEqual('ds_name', ds.name)
         self.assertEqual('fake_ref', ds.ref)
         self.assertEqual(2 * units.Gi, ds.capacity)
         self.assertEqual(1 * units.Gi, ds.freespace)
+        self.assertEqual(1 * units.Gi, ds.uncommitted)
 
     def test_ds_invalid_space(self):
         self.assertRaises(ValueError, datastore.Datastore,
@@ -94,7 +95,8 @@ class DatastoreTestCase(base.TestCase):
                                                    session.vim,
                                                    ds.ref, 'summary')
 
-    def test_get_connected_hosts(self):
+    def _test_get_connected_hosts(self, in_maintenance_mode,
+                                  m1_accessible=True):
         session = mock.Mock()
         ds_ref = vim_util.get_moref('ds-0', 'Datastore')
         ds = datastore.Datastore(ds_ref, 'ds-name')
@@ -102,7 +104,7 @@ class DatastoreTestCase(base.TestCase):
         ds.get_summary.return_value.accessible = False
         self.assertEqual([], ds.get_connected_hosts(session))
         ds.get_summary.return_value.accessible = True
-        m1 = HostMount("m1", MountInfo('readWrite', True, True))
+        m1 = HostMount("m1", MountInfo('readWrite', True, m1_accessible))
         m2 = HostMount("m2", MountInfo('read', True, True))
         m3 = HostMount("m3", MountInfo('readWrite', False, True))
         m4 = HostMount("m4", MountInfo('readWrite', True, False))
@@ -110,12 +112,46 @@ class DatastoreTestCase(base.TestCase):
 
         class Prop(object):
             DatastoreHostMount = [m1, m2, m3, m4]
-        session.invoke_api = mock.Mock()
-        session.invoke_api.return_value = Prop()
+
+        class HostRuntime(object):
+            inMaintenanceMode = in_maintenance_mode
+
+        class HostProp(object):
+            name = 'runtime'
+            val = HostRuntime()
+
+        class Object(object):
+            obj = "m1"
+            propSet = [HostProp()]
+
+        class Runtime(object):
+            objects = [Object()]
+
+        session.invoke_api = mock.Mock(side_effect=[Prop(), Runtime()])
         hosts = ds.get_connected_hosts(session)
+        calls = [mock.call(vim_util, 'get_object_property',
+                           session.vim, ds_ref, 'host')]
+        if m1_accessible:
+            calls.append(
+                mock.call(vim_util,
+                          'get_properties_for_a_collection_of_objects',
+                          session.vim, 'HostSystem', ["m1"], ['runtime']))
+        self.assertEqual(calls, session.invoke_api.mock_calls)
+        return hosts
+
+    def test_get_connected_hosts(self):
+        hosts = self._test_get_connected_hosts(False)
         self.assertEqual(1, len(hosts))
         self.assertEqual("m1", hosts.pop())
 
+    def test_get_connected_hosts_in_maintenance(self):
+        hosts = self._test_get_connected_hosts(True)
+        self.assertEqual(0, len(hosts))
+
+    def test_get_connected_hosts_ho_hosts(self):
+        hosts = self._test_get_connected_hosts(False, False)
+        self.assertEqual(0, len(hosts))
+
     def test_is_datastore_mount_usable(self):
         m = MountInfo('readWrite', True, True)
         self.assertTrue(datastore.Datastore.is_datastore_mount_usable(m))
@@ -133,6 +169,45 @@ class DatastoreTestCase(base.TestCase):
         self.assertFalse(datastore.Datastore.is_datastore_mount_usable(m))
 
 
+class DatastoreClusterTestCase(base.TestCase):
+
+    def test_get_dsc_with_moid(self):
+        session = mock.Mock()
+        session.invoke_api = mock.Mock()
+        session.invoke_api.return_value = 'ds-cluster'
+        dsc_moid = 'group-p123'
+        dsc_ref, dsc_name = datastore.get_dsc_ref_and_name(session, dsc_moid)
+        self.assertEqual((dsc_moid, 'StoragePod'),
+                         (dsc_ref.value, dsc_ref._type))
+        self.assertEqual('ds-cluster', dsc_name)
+        session.invoke_api.assert_called_once_with(vim_util,
+                                                   'get_object_property',
+                                                   session.vim,
+                                                   mock.ANY,
+                                                   'name')
+
+    @mock.patch('oslo_vmware.vim_util.continue_retrieval')
+    @mock.patch('oslo_vmware.vim_util.cancel_retrieval')
+    def test_get_dsc_by_name(self, cancel_retrieval, continue_retrieval):
+        pod_prop = mock.Mock()
+        pod_prop.val = 'ds-cluster'
+        pod_ref = vim_util.get_moref('group-p456', 'StoragePod')
+        pod = mock.Mock()
+        pod.propSet = [pod_prop]
+        pod.obj = pod_ref
+
+        retrieve_result = mock.Mock()
+        retrieve_result.objects = [pod]
+
+        session = mock.Mock()
+        session.invoke_api = mock.Mock()
+        session.invoke_api.return_value = retrieve_result
+        name = 'ds-cluster'
+        dsc_ref, dsc_name = datastore.get_dsc_ref_and_name(session, name)
+        self.assertEqual((pod_ref.value, pod_ref._type),
+                         (dsc_ref.value, dsc_ref._type))
+
+
 class DatastorePathTestCase(base.TestCase):
 
     """Test the DatastorePath object."""
